@@ -173,6 +173,7 @@
 #define QUIT(x)		{fflush(stdout); fprintf x; exit(-1);}
 
 #include "s2types.h"
+#include "byteorder.h"
 #include "CM_macros.h"
 #include "list.h"
 #include "hash.h"
@@ -541,7 +542,8 @@ static void ReadUnigrams (fp, model)
 	model->unigrams[wcnt].prob1.f = p1;
 	model->unigrams[wcnt].bo_wt1.f = bo_wt;
 
-	model->unigrams[wcnt].mapid = wcnt++;
+	model->unigrams[wcnt].mapid = wcnt;
+	++wcnt;
     }
 
     if (model->ucount != wcnt) {
@@ -1235,19 +1237,6 @@ static int32 lm3g2dmp_dictwd_in_lm (wid)
     return (lmp->dictwid_map[wid] >= 0);
 }
 
-#if (__BIG_ENDIAN__)
-
-#define SWAPW(x)	x = ( (((x)<<8)&0x0000ff00) | (((x)>>8)&0x00ff) )
-#define SWAPL(x)	x = ( (((x)<<24)&0xff000000) | (((x)<<8)&0x00ff0000) | \
-    			      (((x)>>8)&0x0000ff00) | (((x)>>24)&0x000000ff) )
-
-#else
-
-#define SWAPW(x)
-#define SWAPL(x)
-
-#endif
-
 static int32 fread_int32(fp, min, max, name)
     FILE *fp;
     int32 min, max;
@@ -1257,7 +1246,7 @@ static int32 fread_int32(fp, min, max, name)
     
     if (fread (&k, sizeof (int32), 1, fp) != 1)
 	QUIT((stderr, "%s(%d): fread(%s) failed\n", __FILE__, __LINE__, name));
-    SWAPL(k);
+    SWAP_LE_32(&k);
     if ((min > k) || (max < k))
 	QUIT((stderr, "%s(%d): %s outside range [%d,%d]\n", __FILE__, __LINE__, name, min, max));
     return (k);
@@ -1267,7 +1256,7 @@ static void fwrite_int32 (fp, val)
     FILE *fp;
     int32 val;
 {
-    SWAPL(val);
+    SWAP_LE_32(&val);
     fwrite (&val, sizeof(int32), 1, fp);
 }
 
@@ -1277,10 +1266,10 @@ static void fwrite_ug (fp, ug)
 {
     unigram_t tmp_ug = *ug;
     
-    SWAPL(tmp_ug.mapid);
-    SWAPL(tmp_ug.prob1.l);
-    SWAPL(tmp_ug.bo_wt1.l);
-    SWAPL(tmp_ug.bigrams);
+    SWAP_LE_32(&tmp_ug.mapid);
+    SWAP_LE_32(&tmp_ug.prob1.l);
+    SWAP_LE_32(&tmp_ug.bo_wt1.l);
+    SWAP_LE_32(&tmp_ug.bigrams);
     fwrite (&tmp_ug, sizeof(unigram_t), 1, fp);
 }
 
@@ -1290,10 +1279,10 @@ static void fwrite_bg (fp, bg)
 {
     bigram_t tmp_bg = *bg;
     
-    SWAPW(tmp_bg.wid);
-    SWAPW(tmp_bg.prob2);
-    SWAPW(tmp_bg.bo_wt2);
-    SWAPW(tmp_bg.trigrams);
+    SWAP_LE_16(&tmp_bg.wid);
+    SWAP_LE_16(&tmp_bg.prob2);
+    SWAP_LE_16(&tmp_bg.bo_wt2);
+    SWAP_LE_16(&tmp_bg.trigrams);
     fwrite (&tmp_bg, sizeof(bigram_t), 1, fp);
 }
 
@@ -1303,8 +1292,8 @@ static void fwrite_tg (fp, tg)
 {
     trigram_t tmp_tg = *tg;
     
-    SWAPW(tmp_tg.wid);
-    SWAPW(tmp_tg.prob3);
+    SWAP_LE_16(&tmp_tg.wid);
+    SWAP_LE_16(&tmp_tg.prob3);
     fwrite (&tmp_tg, sizeof(trigram_t), 1, fp);
 }
 
@@ -1386,10 +1375,10 @@ static int32 lm3g_load (file, model, lmfile, mtime)
     if ((int)fread (model->unigrams, sizeof(unigram_t), model->ucount+1, fp) != model->ucount+1)
 	QUIT((stderr, "%s(%d): fread(unigrams) failed\n", __FILE__, __LINE__));
     for (i = 0, ugptr = model->unigrams; i <= model->ucount; i++, ugptr++) {
-	SWAPL(ugptr->mapid);
-	SWAPL(ugptr->prob1.l);
-	SWAPL(ugptr->bo_wt1.l);
-	SWAPL(ugptr->bigrams);
+	SWAP_LE_32(&ugptr->mapid);
+	SWAP_LE_32(&ugptr->prob1.l);
+	SWAP_LE_32(&ugptr->bo_wt1.l);
+	SWAP_LE_32(&ugptr->bigrams);
     }
     for (i = 0, ugptr = model->unigrams; i < model->ucount; i++, ugptr++) {
 	if (ugptr->mapid != i)
@@ -1406,10 +1395,10 @@ static int32 lm3g_load (file, model, lmfile, mtime)
     if ((int)fread (model->bigrams, sizeof(bigram_t), model->bcount+1, fp) != model->bcount+1)
 	QUIT((stderr, "%s(%d): fread(bigrams) failed\n", __FILE__, __LINE__));
     for (i = 0, bgptr = model->bigrams; i <= model->bcount; i++, bgptr++) {
-	SWAPW(bgptr->wid);
-	SWAPW(bgptr->prob2);
-	SWAPW(bgptr->bo_wt2);
-	SWAPW(bgptr->trigrams);
+	SWAP_LE_16(&bgptr->wid);
+	SWAP_LE_16(&bgptr->prob2);
+	SWAP_LE_16(&bgptr->bo_wt2);
+	SWAP_LE_16(&bgptr->trigrams);
     }
     printf("%s(%d): %8d = LM.bigrams(+trailer) read\n",
 	   __FILE__, __LINE__, model->bcount);
@@ -1419,8 +1408,8 @@ static int32 lm3g_load (file, model, lmfile, mtime)
 	if ((int)fread(model->trigrams, sizeof(trigram_t), model->tcount, fp) != model->tcount)
 	    QUIT((stderr, "%s(%d): fread(trigrams) failed\n", __FILE__, __LINE__));
 	for (i = 0, tgptr = model->trigrams; i < model->tcount; i++, tgptr++) {
-	    SWAPW(tgptr->wid);
-	    SWAPW(tgptr->prob3);
+	    SWAP_LE_16(&tgptr->wid);
+	    SWAP_LE_16(&tgptr->prob3);
 	}
 	printf("%s(%d): %8d = LM.trigrams read\n", __FILE__, __LINE__, model->tcount);
     }
@@ -1431,7 +1420,7 @@ static int32 lm3g_load (file, model, lmfile, mtime)
     if ((int)fread (model->prob2, sizeof (log_t), k, fp) != k)
 	QUIT((stderr, "%s(%d): fread(prob2) failed\n", __FILE__, __LINE__));
     for (i = 0; i < k; i++)
-	SWAPL(model->prob2[i].l);
+	SWAP_LE_32(&model->prob2[i].l);
     printf("%s(%d): %8d = LM.prob2 entries read\n", __FILE__, __LINE__, k);
 
     /* read n_bo_wt2 and bo_wt2 array */
@@ -1442,7 +1431,7 @@ static int32 lm3g_load (file, model, lmfile, mtime)
 	if ((int)fread (model->bo_wt2, sizeof (log_t), k, fp) != k)
 	    QUIT((stderr, "%s(%d): fread(bo_wt2) failed\n", __FILE__, __LINE__));
 	for (i = 0; i < k; i++)
-	    SWAPL(model->bo_wt2[i].l);
+	    SWAP_LE_32(&model->bo_wt2[i].l);
 	printf("%s(%d): %8d = LM.bo_wt2 entries read\n", __FILE__, __LINE__, k);
     }
 	
@@ -1454,7 +1443,7 @@ static int32 lm3g_load (file, model, lmfile, mtime)
 	if ((int)fread (model->prob3, sizeof (log_t), k, fp) != k)
 	    QUIT((stderr, "%s(%d): fread(prob3) failed\n", __FILE__, __LINE__));
 	for (i = 0; i < k; i++)
-	    SWAPL(model->prob3[i].l);
+	    SWAP_LE_32(&model->prob3[i].l);
 	printf("%s(%d): %8d = LM.prob3 entries read\n", __FILE__, __LINE__, k);
     }
     
@@ -1465,7 +1454,7 @@ static int32 lm3g_load (file, model, lmfile, mtime)
 	if ((int)fread (model->tseg_base, sizeof(int32), k, fp) != k)
 	    QUIT((stderr, "%s(%d): fread(tseg_base) failed\n", __FILE__, __LINE__));
 	for (i = 0; i < k; i++)
-	    SWAPL(model->tseg_base[i]);
+	    SWAP_LE_32(&model->tseg_base[i]);
 	printf("%s(%d): %8d = LM.tseg_base entries read\n", __FILE__, __LINE__, k);
     }
 
