@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 1999-2001 Carnegie Mellon University.  All rights
+ * Copyright (c) 1999-2004 Carnegie Mellon University.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,11 +33,24 @@
  * ====================================================================
  *
  */
+
 /*
  * allphone.c -- All CI phone decoding.
  *
  * HISTORY
  * 
+ * $Log: allphone.c,v $
+ * Revision 1.12  2004/12/10 16:48:56  rkm
+ * Added continuous density acoustic model handling
+ *
+ * 
+ * 22-Nov-2004	M K Ravishankar (rkm@cs.cmu.edu) at Carnegie Mellon University
+ * 		Modified to use unified semi-continuous/continuous acoustic
+ * 		model evaluation module (senscr).
+ * 
+ * 06-Aug-2004	M K Ravishankar (rkm@cs.cmu.edu) at Carnegie Mellon University
+ * 		Added phonetp (phone transition probs matrix) to search.
+ * 
  * 10-Sep-98	M K Ravishankar (rkm@cs.cmu.edu) at Carnegie Mellon University
  * 		Modified to let allphone_utt return phone segmentation result,
  * 		instead of simply printing it to the standard log file.
@@ -68,6 +81,7 @@
 #include "err.h"
 #include "log.h"
 #include "scvq.h"
+#include "senscr.h"
 #include "msd.h"
 #include "dict.h"
 #include "hmm_tied_r.h"
@@ -87,6 +101,7 @@ static int32 *renorm_scr;
 static int32 allphone_bw;	/* beam width */
 static int32 allphone_exitbw;	/* phone exit beam width */
 static int32 allphone_pip;	/* phone insertion penalty */
+static int32 **phonetp;		/* phone transition LOGprob */
 
 typedef struct {
     int32 f;
@@ -166,7 +181,8 @@ static int32 allphone_eval_ci_chan (int32 f)
 static void allphone_bp_entry (int32 f, int32 p)
 {
     if (n_bp == ALLPHONE_BP_MAX-2)
-	fprintf (stderr, "%s(%d): **ERROR** BP table full\n", __FILE__, __LINE__);
+	E_ERROR("BP table full\n");
+    
     if (n_bp >= ALLPHONE_BP_MAX)
 	return;
     
@@ -199,18 +215,23 @@ static void allphone_chan_prune (int32 f, int32 bestscr)
 
 static void allphone_chan_trans (int32 f, int32 bp)
 {
-    int32 p, scr, s;
+    int32 p, scr, s, predp;
 
-    scr = allphone_bp[bp].scr + allphone_pip;
+    predp = allphone_bp[bp].p;
+    
     for (p = 0; p < n_ciphone; p++) {
-	if ((ci_chan[p].active < f) || (ci_chan[p].score[0] < scr)) {
-	    ci_chan[p].score[0] = scr;
-	    if (ci_chan[p].active < f)
-		for (s = 1; s < HMM_LAST_STATE; s++)
-		    ci_chan[p].score[s] = WORST_SCORE;
-	    ci_chan[p].path[0] = bp;
-	    ci_chan[p].active = f+1;
-	}
+      scr = allphone_bp[bp].scr + phonetp[predp][p];
+      
+      if ((ci_chan[p].active < f) || (ci_chan[p].score[0] < scr)) {
+	ci_chan[p].score[0] = scr;
+	
+	if (ci_chan[p].active < f)
+	  for (s = 1; s < HMM_LAST_STATE; s++)
+	    ci_chan[p].score[s] = WORST_SCORE;
+	
+	ci_chan[p].path[0] = bp;
+	ci_chan[p].active = f+1;
+      }
     }
 }
 
@@ -345,11 +366,10 @@ search_hyp_t *allphone_utt (int32 nfr,
     renorm_scr[0] = 0;
     
     for (f = 0, c = 0, p = 0; f < nfr; f++, c += CEP_SIZE, p += POW_SIZE) {
-	SCVQScores (senscr, cep+c, dcep+c, dcep_80ms+c, pcep+p, ddcep+c);
+	senscr_active (senscr, cep+c, dcep+c, dcep_80ms+c, pcep+p, ddcep+c);
 
 	if ((bestscr = allphone_eval_ci_chan (f)) <= WORST_SCORE) {
-	    fprintf (stderr, "%s(%d): POOR MATCH: bestscore= %d\n",
-		     __FILE__, __LINE__, bestscr);
+	    E_ERROR ("POOR MATCH: bestscore= %d\n", bestscr);
 	    break;
 	}
 	
@@ -398,6 +418,8 @@ allphone_init (double bw, double exitbw, double pip)
     allphone_exitbw = LOG(exitbw) * 8;
     allphone_pip = LOG(pip);
 
-    printf ("%s(%d): bw= %d, wordbw= %d, pip= %d\n", __FILE__, __LINE__,
-	    allphone_bw, allphone_exitbw, allphone_pip);
+    phonetp = kb_get_phonetp();
+    
+    E_INFO("bw= %d, wordbw= %d, pip= %d\n",
+	   allphone_bw, allphone_exitbw, allphone_pip);
 }
