@@ -70,7 +70,7 @@ areaddouble (char *file, double **data_ref, int *length_ref)
     close (fd);
     return -1;
   }
-  SWAPL(&length);
+  SWAP_BE_32(&length);
   size = length * sizeof (double);
   if (!(data = malloc ((unsigned) size)))
   {
@@ -88,7 +88,7 @@ areaddouble (char *file, double **data_ref, int *length_ref)
   close (fd);
   *data_ref = (double *) data;
   for(offset = 0; offset < length; offset++)
-    SWAPD(*data_ref + offset);
+    SWAP_BE_64(*data_ref + offset);
   *length_ref = length;
   return length;
 }
