@@ -48,6 +48,7 @@
 #include <sys/file.h>
 #include <unistd.h>
 #endif
+#include "s2types.h"
 #include "byteorder.h"
 
 int
@@ -70,7 +71,7 @@ areadfloat (char *file, float **data_ref, int *length_ref)
     close (fd);
     return -1;
   }
-  SWAPL(&length);
+  SWAP_BE_32(&length);
   size = length * sizeof (float);
   if (!(data = malloc ((unsigned) size)))
   {
@@ -88,7 +89,7 @@ areadfloat (char *file, float **data_ref, int *length_ref)
   close (fd);
   *data_ref = (float *) data;
   for(offset = 0; offset < length; offset++)
-    SWAPF(*data_ref + offset);
+    SWAP_BE_32((int32 *)*data_ref + offset);
   *length_ref = length;
   return length;
 }
