@@ -48,6 +48,7 @@
 #include <sys/file.h>
 #include <unistd.h>
 #endif
+#include "s2types.h"
 #include "byteorder.h"
 
 int
@@ -63,17 +64,17 @@ awritefloat (char *file, float *data, int length)
     return -1;
   }
 
-  SWAPL(&length);
+  SWAP_BE_32(&length);
   if (write (fd, (char *) &length, 4) != 4)
   {
     fprintf (stderr, "awritefloat: %s: can't write length\n", file);
     close (fd);
     return -1;
   }
-  SWAPL(&length);
+  SWAP_BE_32(&length);
 
   for(offset = 0; offset < length; offset++)
-    SWAPF(data + offset);
+    SWAP_BE_32((int32 *)data + offset);
   size = length * sizeof (float);
   if (write (fd, (char *) data, size) != size)
   {
@@ -82,7 +83,7 @@ awritefloat (char *file, float *data, int length)
     return (-1);
   }
   for(offset = 0; offset < length; offset++)
-    SWAPF(data + offset);
+    SWAP_BE_32((int32 *)data + offset);
 
   printf ("Wrote %d floats in %s.\n", length, file);
   close (fd);
