@@ -42,15 +42,16 @@
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
+#include "ckd_alloc.h"
 #include "cdcn.h"
 
 int
 cdcn_init(char const *filename, CDCN_type *cdcn_variables)
 {
-    int      i,j,count,count2,ndim,ncodes;
+    int      i,j,ndim,ncodes;
 
-    float    *codebuff;
-    float    *varbuff;
+    float    **codebuff;
+    float    **varbuff;
     float    *probbuff;
     float    temp;
 
@@ -70,15 +71,29 @@ cdcn_init(char const *filename, CDCN_type *cdcn_variables)
         return(-1);
     }
 
-    if (fscanf(codefile,"%d %d",&(cdcn_variables->num_codes),&ndim) == 0)
+    if (fscanf(codefile,"%d %d",&(cdcn_variables->num_codes),&(cdcn_variables->n_dim)) == 0)
     {
         printf("Error in format of cdcn statistics file\n");
         printf("Unable to run CDCN. Will not process cepstra\n");
         cdcn_variables->run_cdcn = FALSE;
         return(-1);
     }
+    ndim = cdcn_variables->n_dim;
+    if (ndim > NUM_COEFF) {
+        printf("Error in data dimension in cdcn statistics file\n");
+        printf("Unable to run CDCN. Will not process cepstra\n");
+        cdcn_variables->run_cdcn = FALSE;
+        return(-1);
+    }
+    if (ndim > N) {
+        printf("Error in data dimension in cdcn statistics file\n");
+        printf("The dimension %d found in the file can be at most %d.\n", ndim, N);
+	printf("Will not process cepstra\n");
+        cdcn_variables->run_cdcn = FALSE;
+        return(-1);
+    }
     ncodes = cdcn_variables->num_codes;
-    codebuff = (float *)malloc(ncodes*ndim*sizeof(float));
+    codebuff = (float **)ckd_calloc_2d(ncodes, ndim, sizeof(float));
     if (codebuff == NULL)
     {
         printf("Unable to allocate space for codebook\n");
@@ -86,7 +101,7 @@ cdcn_init(char const *filename, CDCN_type *cdcn_variables)
         cdcn_variables->run_cdcn = FALSE;
         return(-1);
     }
-    varbuff = (float *)malloc(ndim*ncodes*sizeof(float));
+    varbuff = (float **)ckd_calloc_2d(ncodes, ndim, sizeof(float));
     if (varbuff == NULL)
     {
         printf("Unable to allocate space for variances\n");
@@ -94,7 +109,7 @@ cdcn_init(char const *filename, CDCN_type *cdcn_variables)
         cdcn_variables->run_cdcn = FALSE;
         return(-1);
     }
-    probbuff = (float *)malloc(ncodes*sizeof(float));
+    probbuff = (float *)ckd_malloc(ncodes*sizeof(float));
     if (probbuff == NULL)
     {
         printf("Unable to allocate space for mode probabilites\n");
@@ -102,7 +117,6 @@ cdcn_init(char const *filename, CDCN_type *cdcn_variables)
         cdcn_variables->run_cdcn = FALSE;
         return(-1);
     }
-    count = 0;count2=0;
     for(i=0;i < ncodes ;++i)
     {
         if (fscanf(codefile,"%f",&probbuff[i]) == 0)
@@ -114,27 +128,25 @@ cdcn_init(char const *filename, CDCN_type *cdcn_variables)
         }
         for (j=0;j<ndim;++j)
         {
-            if (fscanf(codefile,"%f",&codebuff[count]) == 0)
+            if (fscanf(codefile,"%f",&codebuff[i][j]) == 0)
             {
                 printf("Error in format of cdcn statistics file\n");
                 printf("Unable to run CDCN. Will not process cepstra\n");
                 cdcn_variables->run_cdcn = FALSE;
                 return(-1);
             }
-            ++count;
         }
 	temp = 1;
         for (j=0;j<ndim;++j)
         {
-            if (fscanf(codefile,"%f",&varbuff[count2]) == 0)
+            if (fscanf(codefile,"%f",&varbuff[i][j]) == 0)
             {
                 printf("Error in format of cdcn statistics file\n");
                 printf("Unable to run CDCN. Will not process cepstra\n");
                 cdcn_variables->run_cdcn = FALSE;
                 return(-1);
             }
-	    temp *= varbuff[count2];
-            ++count2;
+	    temp *= varbuff[i][j];
         }
 	if ((temp = (float)sqrt(temp)) == 0)
         {
@@ -153,7 +165,7 @@ cdcn_init(char const *filename, CDCN_type *cdcn_variables)
     cdcn_variables->firstcall = TRUE;
 
     cdcn_variables->corrbook = 
-                (float *) malloc (ncodes *(NUM_COEFF + 1) * sizeof (float));
+      (float **) ckd_calloc_2d (ncodes, ndim, sizeof (float));
     if (cdcn_variables->corrbook == NULL)
     {
         printf("Unable to allocate space for correction terms\n");
@@ -162,5 +174,21 @@ cdcn_init(char const *filename, CDCN_type *cdcn_variables)
         return(-1);
     }
 
+    cdcn_variables->tilt = (float *)ckd_calloc(ndim, sizeof(float));
+    if (cdcn_variables->tilt == NULL)
+    {
+        printf("Unable to allocate space for tilt vector\n");
+        printf("Unable to run CDCN. Will not process cepstra\n");
+        cdcn_variables->run_cdcn = FALSE;
+        return(-1);
+    }
+    cdcn_variables->noise = (float *)ckd_calloc(ndim, sizeof(float));
+    if (cdcn_variables->noise == NULL)
+    {
+        printf("Unable to allocate space for noise vector\n");
+        printf("Unable to run CDCN. Will not process cepstra\n");
+        cdcn_variables->run_cdcn = FALSE;
+        return(-1);
+    }
     return(0);
 }
