@@ -48,6 +48,7 @@
 #include <sys/file.h>
 #include <unistd.h>
 #endif
+#include "s2types.h"
 #include "byteorder.h"
 
 int
@@ -63,18 +64,18 @@ f2write (char *file, float *data1, float *data2, int length)
     return -1;
   }
 
-  SWAPL(&length);
+  SWAP_BE_32(&length);
   if (write (fd, (char *) &length, 4) != 4)
   {
     fprintf (stderr, "f2write: %s: can't write length\n", file);
     close (fd);
     return -1;
   }
-  SWAPL(&length);
+  SWAP_BE_32(&length);
 
   for(offset = 0; offset < length; offset++) {
-    SWAPF(data1 + offset);
-    SWAPF(data2 + offset);
+    SWAP_BE_32((int32 *)data1 + offset);
+    SWAP_BE_32((int32 *)data2 + offset);
   }
 
   size = length * sizeof (float);
@@ -94,8 +95,8 @@ f2write (char *file, float *data1, float *data2, int length)
   }
 
   for(offset = 0; offset < length; offset++) {
-    SWAPF(data1 + offset);
-    SWAPF(data2 + offset);
+    SWAP_BE_32((int32 *)data1 + offset);
+    SWAP_BE_32((int32 *)data2 + offset);
   }
 
   printf ("Wrote %d * 2 floats in %s.\n", length, file);
