@@ -33,6 +33,7 @@
  * ====================================================================
  *
  */
+
 /* HASH.C
  *------------------------------------------------------------*
  * SYNOPSIS
@@ -52,6 +53,16 @@
  *	It would be trivial to add a deletion function.
  *------------------------------------------------------------*
  * HISTORY
+ * 
+ * $Log: hash.c,v $
+ * Revision 1.12  2005/07/21 20:03:53  egouvea
+ * Fixed bug 1154352, seg fault when extended ascii is the only letter in
+ * the word.
+ *
+ * Revision 1.11  2004/12/10 16:48:56  rkm
+ * Added continuous density acoustic model handling
+ *
+ * 
  * 27-Aug-94  M K Ravishankar (rkm@cs) at Carnegie-Mellon University
  * 	Changed hash_lookup to be case-insensitive.
  * 
@@ -184,7 +195,7 @@ hash_lookup (hash_t *ht, char const *sym, caddr_t *val)
 {
     static char const *rname = "hash_lookup";
     register char const *cp;
-    register char c;
+    register unsigned char c;
     register uint32    key;
     register int32    i;
 
