@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 1999-2001 Carnegie Mellon University.  All rights
+ * Copyright (c) 1999-2004 Carnegie Mellon University.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,9 +33,27 @@
  * ====================================================================
  *
  */
+
 /* 
  * HISTORY
  * 
+ * $Log: search.h,v $
+ * Revision 1.12  2005/05/24 20:55:25  rkm
+ * Added -fsgbfs flag
+ *
+ * Revision 1.11  2005/01/20 15:11:47  rkm
+ * Cleaned up pscr-related functions
+ *
+ * Revision 1.10  2004/12/10 16:48:58  rkm
+ * Added continuous density acoustic model handling
+ *
+ * 
+ * 24-Nov-2004	M K Ravishankar (rkm@cs) at Carnegie Mellon University
+ * 		Added search_get_bestpscr() definition.
+ * 
+ * 12-Aug-2004	M K Ravishankar (rkm@cs) at Carnegie Mellon University
+ * 		Added search_get_current_startwid().
+ * 
  * 02-Aug-95	M K Ravishankar (rkm@cs.cmu.edu) at Carnegie Mellon University
  * 		Changed latnode_t.fef and latnode_t.lef to int32.
  */
@@ -140,6 +158,8 @@ void search_set_hyp_total_score (int32 score);
 void search_set_hyp_total_lscr (int32 lscr);
 void search_set_context (int32 w1, int32 w2);
 void search_set_startword (char const *str);
+int32 search_get_current_startwid ( void );
+
 
 int32 search_result(int32 *fr, char **res);	/* Decoded result as a single string */
 int32 search_partial_result (int32 *fr, char **res);
@@ -168,8 +188,15 @@ BPTBL_T *search_get_bptable ( void );
 void search_postprocess_bptable (double lwf, char const *pass);
 int32 *search_get_bscorestack ( void );
 double search_get_lw ( void );
-uint16 **search_get_uttpscr ( void );
-int32 search_uttpscr2phlat_print ( void );
+int32 **search_get_uttpscr ( void );
+
+/*
+ * Dump a "phone lattice" to the given file:
+ *   For each frame, determine the CIphones with top scoring senones, threshold
+ *   and sort them in descending order.  (Threshold based on topsen_thresh.)
+ */
+int32 search_uttpscr2phlat_print (FILE *outfp);
+
 search_hyp_t *search_uttpscr2allphone ( void );
 void search_remove_context (search_hyp_t *hyp);
 void search_hyp_to_str ( void );
@@ -254,5 +281,24 @@ void search_chan_deactivate(CHAN_T *);
 /* Note the top senone score for the given frame */
 void search_set_topsen_score (int32 frm, int32 score);
 
+/*
+ * Return the array that maintains the best senone score (CI or CD) for
+ * each CI phone.  (The array is updated every frame by the senone score
+ * evaluation module.)
+ */
+int32 *search_get_bestpscr( void );
+
+/* Copy bestpscr to uttpscr[currentframe] */
+void search_bestpscr2uttpscr (int32 currentframe);
+
+/* Reset the uttpscr valid flag */
+void search_uttpscr_reset ( void );
+
+/*
+ * Set the hyp_wid (and n_hyp_wid) global variables in search.c to the given
+ * hypothesis (linked list of search_hyp_t entries).
+ */
+void search_set_hyp_wid (search_hyp_t *hyp);
+
 
 #endif
