@@ -33,6 +33,7 @@
  * ====================================================================
  *
  */
+
 /*
  * linklist.c -- generic module for efficient memory management of 
  * linked list elements of various sizes; a separate list for each 
@@ -41,6 +42,12 @@
  * HISTORY
  * 
  * $Log: linklist.c,v $
+ * Revision 1.10  2005/07/02 12:55:40  egouvea
+ * Changed void* with char* to fix compilation problem when compiling with gcc4.0
+ *
+ * Revision 1.9  2004/12/10 16:48:56  rkm
+ * Added continuous density acoustic model handling
+ *
  * Revision 1.8  2004/05/18 19:02:40  egouvea
  * Removed line including malloc.h, already defined in stdlib.h, and added initial support for macosx
  *
@@ -86,6 +93,7 @@
 #include <stdlib.h>
 
 #include "s2types.h"
+#include "err.h"
 
 #define QUIT(x)		{fprintf x; exit(-1);}
 
@@ -107,7 +115,8 @@ static int32 n_list = 0;
 void *listelem_alloc (int32 elem_size)
 {
     int32 i, j;
-    void **cpp, *cp;
+    void **cpp;
+    char *cp;
     
     for (i = 0; i < n_list; i++) {
 	if (list[i].elem_size == elem_size)
@@ -116,34 +125,35 @@ void *listelem_alloc (int32 elem_size)
     if (i >= n_list) {
 	/* New list element size encountered, create new list entry */
 	if (n_list >= MAX_LIST)
-	    QUIT((stdout, "%s(%d): **ERROR** Increase MAX_LIST\n", __FILE__, __LINE__));
+	    E_FATAL("Increase MAX_LIST\n");
+	
 	if (elem_size > MAX_ALLOC)
-	    QUIT((stdout, "%s(%d): **ERROR** Increase MAX_ALLOC to %d\n",
-		  __FILE__, __LINE__, elem_size));
+	    E_FATAL("Increase MAX_ALLOC to %d\n", elem_size);
+
 	if ((elem_size % sizeof(char *)))
-	    QUIT((stdout, "%s(%d): **ERROR** Element size (%d) not multiple of (char *)\n",
-		  __FILE__, __LINE__, elem_size));
+	    E_FATAL("Element size (%d) not multiple of (char *)\n",
+		    elem_size);
 	
 	list[n_list].freelist = NULL;
 	list[n_list].elem_size = elem_size;
 	list[n_list].n_malloc = MAX_ALLOC / elem_size;
 	i = n_list++;
     }
-    
+
     if (list[i].freelist == NULL) {
 	cpp = list[i].freelist = (void **) malloc (list[i].n_malloc * elem_size);
-	cp = (void *) cpp;
+	cp = (char *) cpp;
 	for (j = list[i].n_malloc-1; j > 0; --j) {
-	    (char*)cp += elem_size;
-	    *cpp = cp;
+	    cp += elem_size;
+	    *cpp = (void *)cp;
 	    cpp = (void **)cp;
 	}
 	*cpp = NULL;
     }
     
-    cp = list[i].freelist;
+    cp = (char *) list[i].freelist;
     list[i].freelist = *(list[i].freelist);
-    return (cp);
+    return ((void *)cp);
 }
 
 void listelem_free (void *elem, int32 elem_size)
@@ -156,8 +166,7 @@ void listelem_free (void *elem, int32 elem_size)
 	    break;
     }
     if (i >= n_list)
-	QUIT((stdout, "%s(%d): **ERROR** elem_size (%d) not in known list\n",
-	      __FILE__, __LINE__, elem_size));
+	E_FATAL("elem_size (%d) not in known list\n", elem_size);
     
     cpp = elem;
     *cpp = list[i].freelist;
