@@ -33,11 +33,17 @@
  * ====================================================================
  *
  */
+
 /*
  * live_norm.c - feature normalization for live system
  * 
  * HISTORY
  * 
+ * $Log: live_norm.c,v $
+ * Revision 1.10  2004/12/10 16:48:56  rkm
+ * Added continuous density acoustic model handling
+ *
+ * 
  * 20-Aug-98	M K Ravishankar (rkm@cs.cmu.edu) at Carnegie Mellon University
  * 		Added c0 to cepmean_set() and cepmean_get().  Changed initialization
  * 		to a more reasonable value for c0.
@@ -51,7 +57,7 @@
 
 /*
  |------------------------------------------------------------*
- | $Header: /cvsroot/cmusphinx/sphinx2/src/libsphinx2/live_norm.c,v 1.9 2004/07/16 00:57:11 egouvea Exp $
+ | $Header: /cvsroot/cmusphinx/sphinx2/src/libsphinx2/live_norm.c,v 1.10 2004/12/10 16:48:56 rkm Exp $
  |------------------------------------------------------------*
  | Description
  |
