@@ -33,11 +33,23 @@
  * ====================================================================
  *
  */
+
 /*
  * searchlat.c -- construct word lattice and search all paths for best path.
  * 
  * HISTORY
  * 
+ * $Log: searchlat.c,v $
+ * Revision 1.13  2005/09/21 02:18:06  rkm
+ * Fixed altpron bug in nbest
+ *
+ * Revision 1.12  2004/12/10 16:48:57  rkm
+ * Added continuous density acoustic model handling
+ *
+ * 
+ * 12-Aug-2004	M K Ravishankar (rkm@cs) at Carnegie Mellon University
+ * 		Bugfix: Obtained current start_wid at the start of lattice_rescore().
+ * 
  * 03-Apr-98	M K Ravishankar (rkm@cs) at Carnegie Mellon University
  * 		Added searchlat_set_rescore_lm().
  * 
@@ -480,7 +492,7 @@ static int32 build_lattice (int32 bptbl_sz)
 	    break;
     }
     if (! node) {
-	E_ERROR("%s(%d): Couldn't find <s>.0\n", __FILE__, __LINE__);
+	E_ERROR("Couldn't find <s>.0\n");
 	return (0);
     }
     start_node = node;
@@ -492,7 +504,7 @@ static int32 build_lattice (int32 bptbl_sz)
 	    break;
     }
     if (! node) {
-	E_ERROR("%s(%d): Couldn't find </s>.%d\n", __FILE__, __LINE__, last_frame);
+	E_ERROR("Couldn't find </s>.%d\n", last_frame);
 	return 0;
     }
     final_node = node;
@@ -544,7 +556,7 @@ static int32 build_lattice (int32 bptbl_sz)
     
     /* There must be at least one path between <s>.0 and </s>.last_frame */
     if (! start_node->reachable) {
-	E_ERROR ("%s(%d): <s> unreachable\n", __FILE__, __LINE__);
+	E_ERROR ("<s>.0 isolated; unreachable\n");
 	return (0);
     }
 
@@ -712,7 +724,8 @@ static void lattice_seg_back_trace (latlink_t *link)
 
 	if (ISA_REAL_WORD(link->from->wid)) {
 	    if (seg >= HYP_SZ-1)
-		QUIT((stdout, "%s(%d): **ERROR** Increase HYP_SZ\n", __FILE__, __LINE__));
+		E_FATAL("**ERROR** Increase HYP_SZ\n");
+	    
 	    hyp[seg].wid = link->from->wid;
 	    hyp[seg].sf = uttproc_feat2rawfr(link->from->sf);
 	    hyp[seg].ef = uttproc_feat2rawfr(link->ef);
@@ -777,6 +790,8 @@ int32 lattice_rescore ( double lwf )
     filler_pen = search_get_filler_penalty ();
     lw_factor = lwf;
     
+    start_wid = search_get_current_startwid();
+    
     if (latnode_list) {
 	destroy_lattice (latnode_list);
 	latnode_list = NULL;
@@ -1057,6 +1072,7 @@ static int32 best_rem_score (latnode_t *from)
 {
     latlink_t *link;
     int32 bestscore, score;
+    int32 bw1, bw2;
     
     if (from->info.rem_score <= 0)
 	return (from->info.rem_score);
@@ -1066,7 +1082,16 @@ static int32 best_rem_score (latnode_t *from)
     for (link = from->links; link; link = link->next) {
 	score = best_rem_score (link->to);
 	score += link->link_scr;
-	score += (lm_bg_score (from->wid, link->to->wid) * lw_factor);
+	
+	if (altpron) {
+	  bw1 = dict->dict_list[from->wid]->fwid;
+	  bw2 = dict->dict_list[link->to->wid]->fwid;
+	  
+	  score += (lm_bg_score (bw1, bw2) * lw_factor);
+	} else {
+	  score += (lm_bg_score (from->wid, link->to->wid) * lw_factor);
+	}
+	
 	if (score > bestscore)
 	    bestscore = score;
     }
@@ -1128,6 +1153,7 @@ static void  path_extend (latpath_t *path)
     latlink_t *link;
     latpath_t *newpath;
     int32 total_score, tail_score;
+    int32 bw0, bw1, bw2;
     
     /* Consider all successors of path->node */
     for (link = path->node->links; link; link = link->next) {
@@ -1140,13 +1166,30 @@ static void  path_extend (latpath_t *path)
 	newpath->node = link->to;
 	newpath->parent = path;
 	newpath->score = path->score + link->link_scr;
-	if (path->parent)
+	if (path->parent) {
+	  if (altpron) {
+	    bw0 = dict->dict_list[path->parent->node->wid]->fwid;
+	    bw1 = dict->dict_list[path->node->wid]->fwid;
+	    bw2 = dict->dict_list[newpath->node->wid]->fwid;
+	    
+	    newpath->score += (lm_tg_score (bw0, bw1, bw2) * lw_factor);
+	  } else {
 	    newpath->score += (lm_tg_score (path->parent->node->wid,
-					   path->node->wid,
-					   newpath->node->wid) * lw_factor);
-	else
-	    newpath->score += (lm_bg_score (path->node->wid, newpath->node->wid) *
-			       lw_factor);
+					    path->node->wid,
+					    newpath->node->wid)
+			       * lw_factor);
+	  }
+	} else {
+	  if (altpron) {
+	    bw1 = dict->dict_list[path->node->wid]->fwid;
+	    bw2 = dict->dict_list[newpath->node->wid]->fwid;
+	    
+	    newpath->score += (lm_bg_score (bw1, bw2) * lw_factor);
+	  } else {
+	    newpath->score += (lm_bg_score (path->node->wid, newpath->node->wid)
+			       * lw_factor);
+	  }
+	}
 	
 	/* Insert new partial path hypothesis into sorted path_list */
 	n_hyp_tried++;
@@ -1200,7 +1243,7 @@ int32 search_get_alt (int32 n,			/* In: No. of alternatives to look for */
 {
     latnode_t *node;
     latpath_t *path, *top;
-    int32 i, j, scr, n_alt;
+    int32 i, j, scr, n_alt, bwid;
     static search_hyp_t **alt = NULL;
     static int32 max_alt_hyp = 0;
     char remLMName[128];
@@ -1261,7 +1304,15 @@ int32 search_get_alt (int32 n,			/* In: No. of alternatives to look for */
 	    path = (latpath_t *) listelem_alloc (sizeof(latpath_t));
 	    path->node = node;
 	    path->parent = NULL;
-	    scr = (w1 < 0) ? lm_bg_score(w2, node->wid) : lm_tg_score(w1, w2, node->wid);
+	    
+	    bwid = (altpron) ? dict->dict_list[node->wid]->fwid : node->wid;
+	    
+	    if (w1 < 0) {
+	      scr = lm_bg_score(w2, bwid);
+	    } else {
+	      scr = lm_tg_score(w1, w2, bwid);
+	    }
+	    
 	    path->score = scr;
 	    
 	    path_insert (path, scr + node->info.rem_score);
@@ -1311,9 +1362,9 @@ int32 search_get_alt (int32 n,			/* In: No. of alternatives to look for */
     }
 
 #if defined(SEARCH_PROFILE)
-    printf ("%s(%d): #HYP = %d, #INSERT = %d, #REJECT = %d, Avg insert depth = %.2f\n",
-	    __FILE__, __LINE__, n_hyp_tried, n_hyp_insert, n_hyp_reject,
-	    insert_depth/((double)n_hyp_insert+1.0));
+    E_INFO("#HYP = %d, #INSERT = %d, #REJECT = %d, Avg insert depth = %.2f\n",
+	   n_hyp_tried, n_hyp_insert, n_hyp_reject,
+	   insert_depth/((double)n_hyp_insert+1.0));
 #endif
     
     /* Clean up */
